using System;
using System.Net;
using ESRI.BAServer.SOAP.BAServerProxy;

namespace ESRI.BAServer.SOAP
{
	/// <summary>
	/// The helper class to the DefaultMap_BAServer client class for Business Analyst Server 10.0.
	/// </summary>
	/// <remarks>
	/// <para>
	/// This helper class works with both secured and non-secured instances of the Business Analyst Server.
	/// If the security is required, an instance of this class should be constructed with
	/// a token provider. The token provider is responsible for requesting a new authentication
	/// token when a previous token expires. The authentication token is passed to the Business Analyst Server
	/// in the query of URL as the "token=value" parameter.
	/// </para>
	/// <para>
	/// Every method of the DefaultMap_BAServer client is executed within the try-catch block and,
	/// if the security is required and the token is expired, a new token is automatically requested,
	/// the client URL is updated, and the method is executed once more. 
	/// </para>
	/// <para>
	/// The proxy-methods of this class simplify the work with the Business Analyst Server
	/// by applying some pre-processing and post-processing actions:
	/// <list type="bullet">
	/// <item><description>
	/// The common parameters of tasks, <c>ActiveDataSetID</c> and <c>IsFullErrorMessage</c>,
	/// are specified in properties
	/// of this class and are automatically updated in parameters of a task using the
	/// <c>SetAnalysisParameters</c> pre-processing method.
	/// </description></item>
	/// <item><description>
	/// Server name in all URLs returned in the results
	/// of proxy-methods is updated to the fully-addressable server name if the <c>BAServerDomain</c> property
	/// is specified. This work is provided with the <c>UpdateLocalUrl</c> post-processing method.
	/// </description></item>
	/// </list>
	/// <note type="caution">
	/// The signature of some proxy-methods of this class differs from the signature of corresponding
	/// methods of the DefaultMap_BAServer class: deprecated parameters are removed and the ActiveDatasetID
	/// parameter is absent in parameters of proxy-methods. Also, in methods producing reports only, the
	/// <c>OutputType[]</c> parameter is removed because it is redundant.
	/// </note>
	/// </para>
	/// </remarks>
	public class BAServerHelper
	{
		DefaultMap_BAServer baServer = new DefaultMap_BAServer();
		ITokenProvider tokenProvider = null;

		string domain = null;            // Business Analyst Server domain name
		string activeDatasetID = null;   // Active dataset ID
		bool isFullErrorMessage = false; // Error message mode for AnalysisParameters

		#region Constructors

		/// <overloads>Initializes a new instance of the <see cref="BAServerHelper"/> class.</overloads>
		/// <summary>
		/// Initializes a new non-secured instance of the <see cref="BAServerHelper"/> class.
		/// </summary>
		public BAServerHelper() { }

		/// <summary>
		/// Initializes a new instance of the <see cref="BAServerHelper"/> class by the given token provider.
		/// </summary>
		/// <remarks>If the <paramref name="tokenProvider"/> parameter is null, a non-secured instance is created.</remarks>
		/// <param name="tokenProvider">A provider of an authentication token or null.</param>
		public BAServerHelper(ITokenProvider tokenProvider) : this(null, tokenProvider) { }

		/// <summary>
		/// Initializes a new instance of the <see cref="BAServerHelper"/> class by the given URL and token provider.
		/// </summary>
		///<remarks>If the <paramref name="url"/> parameter is null, the BAServer URL is unchanged.
		/// If the <paramref name="tokenProvider"/> parameter is null, a non-secured instance is created.</remarks>
		/// <param name="url">Business Analyst web services URL or null.</param>
		/// <param name="tokenProvider">A provider of an authentication token or null.</param>
		public BAServerHelper(string url, ITokenProvider tokenProvider)
		{
			this.tokenProvider = tokenProvider;
			if (url != null) baServer.Url = url;
			if (tokenProvider != null) SetToken();
			baServer.EnableDecompression = true;
		}

		void SetToken()
		{
			UriBuilder uriBuilder = new UriBuilder(baServer.Url);
			string token = tokenProvider.CurrentToken;
			uriBuilder.Query = string.IsNullOrEmpty(token) ? null : "token=" + token;
			baServer.Url = uriBuilder.Uri.ToString();
		}

		#endregion

		#region Common Members

		/// <summary>
		/// Gets an instance of the DefaultMap_BAServer client associated with this helper.
		/// </summary>
		public DefaultMap_BAServer Client { get { return baServer; } }

		/// <summary>
		/// Gets or sets a URL for the DefaultMap_BAServer client associated with this helper.
		/// </summary>
		/// <remarks>
		/// The getter method returns "clear" client URL without query parameters (without authentication token).
		/// The setter method assigns URL to the client and appends if necessary the authentication token.
		/// </remarks>
		public string Url
		{
			get
			{
				UriBuilder uriBuilder = new UriBuilder(baServer.Url);
				uriBuilder.Query = null; // Remove the "token" parameter from URL
				return uriBuilder.Uri.ToString();
			}
			set
			{
				baServer.Url = value;
				if (tokenProvider != null) SetToken();
			}
		}

		/// <summary>
		/// Sets the Business Analyst Server domain name.
		/// </summary>
		/// <remarks>The domain name is used in the UpdateLocalUrl method.</remarks>
		public string BAServerDomain
		{
			set { domain = string.IsNullOrEmpty(value) ? null : "." + value; }
		}

		/// <summary>
		/// Gets or sets the Active Dataset ID.
		/// </summary>
		public string ActiveDatasetID
		{
			get { return activeDatasetID; }
			set { activeDatasetID = value; }
		}

		/// <summary>
		/// Sets the error message mode to be specified in the analysis parameters.
		/// </summary>
		public bool IsFullErrorMessage { set { isFullErrorMessage = value; } }

		/// <summary>
		/// Sets values for analysis parameters.
		/// </summary>
		/// <remarks>If <c>ActiveDatasetID</c> is missing in the instance of
		/// parameters class, the <c>ActiveDatasetID</c> property value of this class instance
		/// is copied to the parameters class instance.</remarks>
		/// <param name="parameters">Analysis parameters.</param>
		public void SetAnalysisParameters(AnalysisParameters parameters)
		{
			if (string.IsNullOrEmpty(parameters.ActiveDatasetID))
				parameters.ActiveDatasetID = activeDatasetID;
			parameters.IsFullErrorMessage = isFullErrorMessage;
		}

		/// <summary>
		/// Tests the exception status code to be the "Token Expired" event.
		/// </summary>
		/// <param name="ex">Web exception.</param>
		/// <returns>True, if the security is enabled and the "Token Expired" event is recognized.</returns>
		public bool IsTokenExpired(WebException ex)
		{
			if (tokenProvider == null) return false;

			HttpWebResponse response = (HttpWebResponse)ex.Response;
			return (int)response.StatusCode == 498;
		}

		/// <summary>
		/// Updates the authentication token in the DefaultMap_BAServer client URL
		/// if the current instance of Business Analyst Server is secured.
		/// </summary>
		public void UpdateToken()
		{
			if (tokenProvider != null)
			{
				tokenProvider.UpdateToken();
				SetToken();
			}
		}

		/// <summary>
		/// Updates the local URL by adding the domain name if necessary.
		/// </summary>
		/// <remarks>
		/// If necessary, this method replaces your local server name with the fully-addressable server name
		/// so people and machines outside the domain of the local Business Analyst Server instance
		/// can access the URL. This code will only perform the replacement if the server address
		/// in URL only contains a server name without the domain information and the BAServerDomain property
		/// is specified.
		/// </remarks>
		/// <param name="url">URL to be updated (the null reference is allowed).</param>
		/// <returns>The updated URL or the original URL.</returns>
		public string UpdateLocalUrl(string url)
		{
			if (url == null || domain == null) return url;

			UriBuilder uriBuilder = new UriBuilder(url);
			if (!uriBuilder.Host.Contains("."))
			{
				uriBuilder.Host += domain;
				url = uriBuilder.Uri.ToString();
			}
			return url;
		}

		/// <summary>
		/// Updates all local URLs in a task result output to URLs with fully-addressable server name.
		/// </summary>
		/// <param name="output">Task result output.</param>
		public void UpdateLocalUrls(TaskResultOutput output)
		{
			if (domain == null) return;

			// Update MapImage URL if the output contains it.
			MapImage mapImage = output.MapImage;
			if (mapImage != null)
				mapImage.ImageURL = UpdateLocalUrl(mapImage.ImageURL);

			// Update report URLs if the output contains reports.
			ReportInfo[] reports = output.Reports;
			if (reports != null)
			{
				foreach (ReportInfo report in reports)
					report.ReportURL = UpdateLocalUrl(report.ReportURL);
			}
		}
		#endregion

		#region Static Helper Members

		/// <summary>
		/// Internal constant used in tasks producing a report only.
		/// </summary>
		static TaskOutputType[] ReportOutput = { TaskOutputType.GetReport };

		#endregion

		#region Proxy-methods for Analyses Tasks

		/// <summary>
		/// Profiles some demographic attributes of customer data calculating statistical parameters which
		/// are floor and ceiling bounds, an average, and a standard deviation.
		/// </summary>
		/// <param name="parameters">Customer profiling options.</param>
		/// <returns>Customer Profile string in XML format.</returns>
		public string CustomerProfiling(CustomerProfilingParameters parameters)
		{
			SetAnalysisParameters(parameters);
			try
			{
				return Client.CustomerProfiling(parameters);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.CustomerProfiling(parameters);

			}
		}

		/// <summary>
		/// Locates areas with nearest demography characteristics to the given profile.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated <c>esriReportFormat</c> parameter is removed from this proxy-method.
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="imageOptions">Configuration options for rendering the output map image.</param>
		/// <param name="outputOptions">Array of task output options. Options for this method include
		/// rendering the output image (GetMapImage)
		/// and creating the feature class for subsequent analysis (GetFeatureClass).</param>
		/// <param name="outputAnalysisItem">Optional output repository item to save the analysis.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput CustomerProspecting(CustomerProspectingParameters parameters,
			RenderingParameters imageOptions, TaskOutputType[] outputOptions, esriFolderItem outputAnalysisItem)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.CustomerProspecting(parameters, null, imageOptions, outputOptions, outputAnalysisItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.CustomerProspecting(parameters, null, imageOptions, outputOptions, outputAnalysisItem);
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Creates desire lines (spider diagrams) drawn from every customer to associated store location.
		/// </summary>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="imageOptions">Configuration options for rendering the output map image.</param>
		/// <param name="outputOptions">Array of task output options. Options for this method include
		/// rendering the output image (GetMapImage)
		/// and creating the feature class for subsequent analysis (GetFeatureClass).</param>
		/// <param name="outputAnalysisItem">Optional output repository item to save the analysis.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput DesireLines(DesireLinesParameters parameters,
			RenderingParameters imageOptions, TaskOutputType[] outputOptions, esriFolderItem outputAnalysisItem)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.DesireLines(parameters, imageOptions, outputOptions, outputAnalysisItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.DesireLines(parameters, imageOptions, outputOptions, outputAnalysisItem);
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Scores potential sites or trade areas against an existing well-performing master site.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated <c>esriReportFormat</c> parameter is removed from this proxy-method.
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="imageOptions">Configuration options for rendering the output map image.</param>
		/// <param name="outputOptions">Array of task output options. Options for this method include
		/// rendering the output image (GetMapImage)
		/// and creating the feature class for subsequent analysis (GetFeatureClass).</param>
		/// <param name="outputAnalysisItem">Optional output repository item to save the analysis.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput FindSimilar(FindSimilarParameters parameters,
			RenderingParameters imageOptions, TaskOutputType[] outputOptions, esriFolderItem outputAnalysisItem)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.FindSimilar(parameters, null, imageOptions, outputOptions, outputAnalysisItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.FindSimilar(parameters, null, imageOptions, outputOptions, outputAnalysisItem);
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Aggregates data from a geographic layer with another layer.
		/// </summary>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="imageOptions">Configuration options for rendering the output map image.</param>
		/// <param name="outputOptions">Array of task output options. Options for this method include
		/// rendering the output image (GetMapImage)
		/// and creating the feature class for subsequent analysis (GetFeatureClass).</param>
		/// <param name="outputAnalysisItem">Optional output repository item to save the analysis.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput SpatialOverlay(SpatialOverlayParameters parameters,
			RenderingParameters imageOptions, TaskOutputType[] outputOptions, esriFolderItem outputAnalysisItem)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.SpatialOverlay(parameters, imageOptions, outputOptions, outputAnalysisItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.SpatialOverlay(parameters, imageOptions, outputOptions, outputAnalysisItem);
			}
			UpdateLocalUrls(output);
			return output;
		}

		#endregion

		#region Proxy-methods for Customer/Store Setup Tasks

		/// <summary>
		/// Assigns or reassigns customer features to stores using straight-line distance to the closest store feature location.
		/// </summary>
		/// <param name="parameters">Configuration options for customers assignment.</param>
		/// <param name="imageOptions">Configuration options for rendering the output map image.</param>
		/// <param name="outputOptions">Array of task output options. Options for this method include
		/// rendering the output image (GetMapImage)
		/// and creating the feature class for subsequent analysis (GetFeatureClass).</param>
		/// <param name="outputAnalysisItem">Optional output repository item to save the analysis.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput AssignCustomersByClosestStore(AssignByClosestStoreParameters parameters,
			RenderingParameters imageOptions, TaskOutputType[] outputOptions, esriFolderItem outputAnalysisItem)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.AssignCustomersByClosestStore(parameters, imageOptions, outputOptions, outputAnalysisItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.AssignCustomersByClosestStore(parameters, imageOptions, outputOptions, outputAnalysisItem);
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Assigns customers to stores by using existing trade areas.
		/// </summary>
		/// <param name="parameters">Configuration options for customers assignment.</param>
		/// <param name="imageOptions">Configuration options for rendering the output map image.</param>
		/// <param name="outputOptions">Array of task output options. Options for this method include
		/// rendering the output image (GetMapImage)
		/// and creating the feature class for subsequent analysis (GetFeatureClass).</param>
		/// <param name="outputAnalysisItem">Optional output repository item to save the analysis.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput AssignCustomersByTradeArea(AssignByTradeAreaParameters parameters,
			RenderingParameters imageOptions, TaskOutputType[] outputOptions, esriFolderItem outputAnalysisItem)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.AssignCustomersByTradeArea(parameters, imageOptions, outputOptions, outputAnalysisItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.AssignCustomersByTradeArea(parameters, imageOptions, outputOptions, outputAnalysisItem);
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Creates store or customer records by geocoding their postal addresses.
		/// </summary>
		/// <param name="addressRecords">Array of address records.</param>
		/// <param name="folderType">Records type (<c>esriFolderType.esriFolderStoreLayers</c> or
		/// <c>esriFolderType.esriFolderCustomerLayers</c>).</param>
		/// <returns>The task result output whose <c>RecordSet</c> property contains geocoded records.</returns>
		public TaskResultOutput CreateRecordSetByAddresses(RecordSetByAddress[] addressRecords,
			esriFolderType folderType)
		{
			try
			{
				return Client.CreateRecordSetByAddresses(addressRecords, folderType);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.CreateRecordSetByAddresses(addressRecords, folderType);
			}
		}

		/// <summary>
		/// Creates a store or customer layer from records containing latitude (y) and longitude (x) information.
		/// </summary>
		/// <param name="parameters">Configuration options for customer/store setup.</param>
		/// <param name="imageOptions">Configuration options for rendering the output map image.</param>
		/// <param name="outputOptions">Array of task output options. Options for this method include
		/// rendering the output image (GetMapImage)
		/// and creating the feature class for subsequent analysis (GetFeatureClass).</param>
		/// <param name="outputAnalysisItem">Optional output repository item to save the analysis.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput CustomerStoreSetupByCoordinates(CustomerStoreSetupByCoordinatesParameters parameters,
			RenderingParameters imageOptions, TaskOutputType[] outputOptions, esriFolderItem outputAnalysisItem)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.CustomerStoreSetupByCoordinates(parameters, imageOptions, outputOptions, outputAnalysisItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.CustomerStoreSetupByCoordinates(parameters, imageOptions, outputOptions, outputAnalysisItem);
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Creates a store or customer layer from a table containing street address information.
		/// </summary>
		/// <param name="parameters">Configuration options for customer/store setup.</param>
		/// <param name="imageOptions">Configuration options for rendering the output map image.</param>
		/// <param name="outputOptions">Array of task output options. Options for this method include
		/// rendering the output image (GetMapImage)
		/// and creating the feature class for subsequent analysis (GetFeatureClass).</param>
		/// <param name="outputAnalysisItem">Optional output repository item to save the analysis.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput CustomerStoreSetupByGeocodeTable(CustomerStoreSetupByGeocodeTableParameters parameters,
			RenderingParameters imageOptions, TaskOutputType[] outputOptions, esriFolderItem outputAnalysisItem)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.CustomerStoreSetupByGeocodeTable(parameters, imageOptions, outputOptions, outputAnalysisItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.CustomerStoreSetupByGeocodeTable(parameters, imageOptions, outputOptions, outputAnalysisItem);
			}
			UpdateLocalUrls(output);
			return output;
		}

		#endregion

		#region Proxy-methods for Report Tasks

		/// <summary>
		/// Calculates the average drive time within trade areas.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated <c>esriReportFormat</c> parameter is removed from this proxy-method.
		/// The signature of this proxy-method is also simplified by removing the <c>OutputType[]</c> parameter
		/// which always contains the <c>OutputType.GetReport</c> option only.
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="outputReportItem">Optional output repository item to save the report.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput AverageDriveTimeReport(AverageDriveTimeReportParameters parameters,
			esriFolderItem outputReportItem)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.AverageDriveTimeReport(parameters, null, ReportOutput, outputReportItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.AverageDriveTimeReport(parameters, null, ReportOutput, outputReportItem);
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// A comparative report that benchmarks two or more trade areas based on selected volumetric data
		/// (typically demographic data).
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated <c>esriReportFormat</c> parameter is removed from this proxy-method.
		/// The signature of this proxy-method is also simplified by removing the <c>OutputType[]</c> parameter
		/// which always contains the <c>OutputType.GetReport</c> option only.
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="outputReportItem">Optional output repository item to save the report.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput BenchmarkReport(BenchmarkReportParameters parameters,
			esriFolderItem outputReportItem)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.BenchmarkReport(parameters, null, ReportOutput, outputReportItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.BenchmarkReport(parameters, null, ReportOutput, outputReportItem);
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Summarizes the demographic makeup of the geographies where your customers live.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated <c>esriReportFormat</c> parameter is removed from this proxy-method.
		/// The signature of this proxy-method is also simplified by removing the <c>OutputType[]</c> parameter
		/// which always contains the <c>OutputType.GetReport</c> option only.
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="outputReportItem">Optional output repository item to save the report.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput CustomerDemographicProfile(CustomerDemographicProfileParameters parameters,
			esriFolderItem outputReportItem)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.CustomerDemographicProfile(parameters, null, ReportOutput, outputReportItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.CustomerDemographicProfile(parameters, null, ReportOutput, outputReportItem);
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Compares the demographic profiles of two customer layers and reports on the differences.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated <c>esriReportFormat</c> parameter is removed from this proxy-method.
		/// The signature of this proxy-method is simplified by removing the <c>OutputType[]</c> parameter
		/// which always contains the <c>OutputType.GetReport</c> option only.
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="outputReportItem">Optional output repository item to save the report.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput CustomerDemographicComparison(CustomerDemographicComparisonParameters parameters,
			esriFolderItem outputReportItem)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.CustomerDemographicComparison(parameters, null, ReportOutput, outputReportItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.CustomerDemographicComparison(parameters, null, ReportOutput, outputReportItem);
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Creates a report containing the geography areas where most of customers are located,
		/// ranked top to bottom according to the number of customers in each area.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated <c>esriReportFormat</c> parameter is removed from this proxy-method.
		/// The signature of this proxy-method is also simplified by removing the <c>OutputType[]</c> parameter
		/// which always contains the <c>OutputType.GetReport</c> option only.
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="outputReportItem">Optional output repository item to save the report.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput CustomerGeographicSummary(CustomerGeographicSummaryParameters parameters,
			esriFolderItem outputReportItem)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.CustomerGeographicSummary(parameters, null, ReportOutput, outputReportItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.CustomerGeographicSummary(parameters, null, ReportOutput, outputReportItem);
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Calculates the market penetration for ring or drive-time or drive-distance polygons around stores.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated <c>esriReportFormat</c> parameter is removed from this proxy-method.
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="imageOptions">Configuration options for rendering the output map image.</param>
		/// <param name="outputOptions">Array of task output options. Options for this method include rendering
		/// output image (GetMapImage), creating the feature class for subsequent analysis (GetFeatureClass),
		/// and creating a report (GetReport).</param>
		/// <param name="outputAnalysisItem">Optional output repository item to save the analysis.</param>
		/// <param name="outputReportItem">Optional output repository item to save the report.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput DistanceDecay(DistanceDecayParameters parameters, RenderingParameters imageOptions,
			TaskOutputType[] outputOptions, esriFolderItem outputAnalysisItem, esriFolderItem outputReportItem)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.DistanceDecay(parameters, null, imageOptions, outputOptions,
					outputAnalysisItem, outputReportItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.DistanceDecay(parameters, null, imageOptions, outputOptions,
				   outputAnalysisItem, outputReportItem);
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Generates a report based on the relation of stores to competitors.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated <c>esriReportFormat</c> parameter is removed from this proxy-method.
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="imageOptions">Configuration options for rendering the output map image.</param>
		/// <param name="outputOptions">Array of task output options. Options for this method include rendering
		/// output image (GetMapImage), creating the feature class for subsequent analysis (GetFeatureClass),
		/// and creating a report (GetReport).</param>
		/// <param name="outputAnalysisItem">Optional output repository item to save the analysis.</param>
		/// <param name="outputReportItem">Optional output repository item to save the report.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput LocatorReport(LocatorReportParameters parameters, RenderingParameters imageOptions,
			TaskOutputType[] outputOptions, esriFolderItem outputAnalysisItem, esriFolderItem outputReportItem)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.LocatorReport(parameters, null, imageOptions, outputOptions,
					outputAnalysisItem, outputReportItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.LocatorReport(parameters, null, imageOptions, outputOptions,
				   outputAnalysisItem, outputReportItem);
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Calculates the market penetration based on the customer data within an area.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated <c>esriReportFormat</c> parameter is removed from this proxy-method.
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="imageOptions">Configuration options for rendering the output map image.</param>
		/// <param name="outputOptions">Array of task output options. Options for this method include rendering
		/// output image (GetMapImage), creating the feature class for subsequent analysis (GetFeatureClass),
		/// and creating a report (GetReport).</param>
		/// <param name="outputAnalysisItem">Optional output repository item to save the analysis.</param>
		/// <param name="outputReportItem">Optional output repository item to save the report.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput MarketPenetration(MarketPenetrationParameters parameters, RenderingParameters imageOptions,
			TaskOutputType[] outputOptions, esriFolderItem outputAnalysisItem, esriFolderItem outputReportItem)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.MarketPenetration(parameters, null, imageOptions, outputOptions,
					outputAnalysisItem, outputReportItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.MarketPenetration(parameters, null, imageOptions, outputOptions,
				   outputAnalysisItem, outputReportItem);
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Generates a report to rank data based on a selected variable.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated <c>esriReportFormat</c> parameter is removed from this proxy-method.
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="imageOptions">Configuration options for rendering the output map image.</param>
		/// <param name="outputOptions">Array of task output options. Options for this method include rendering
		/// output image (GetMapImage), creating the feature class for subsequent analysis (GetFeatureClass),
		/// and creating a report (GetReport).</param>
		/// <param name="outputAnalysisItem">Optional output repository item to save the analysis.</param>
		/// <param name="outputReportItem">Optional output repository item to save the report.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput MarketRanking(MarketRankingParameters parameters, RenderingParameters imageOptions,
			TaskOutputType[] outputOptions, esriFolderItem outputAnalysisItem, esriFolderItem outputReportItem)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.MarketRanking(parameters, null, imageOptions, outputOptions,
					outputAnalysisItem, outputReportItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.MarketRanking(parameters, null, imageOptions, outputOptions,
				   outputAnalysisItem, outputReportItem);
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Creates a report that assigns customer records to one of five match levels based on the geographic code
		/// assigned to each customer while geocoding.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated <c>esriReportFormat</c> parameter is removed from this proxy-method.
		/// The signature of this proxy-method is also simplified by removing the <c>OutputType[]</c> parameter
		/// which always contains the <c>OutputType.GetReport</c> option only.
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="outputReportItem">Optional output repository item to save the report.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput MatchLevelSummaryReport(MatchLevelSummaryReportParameters parameters,
			esriFolderItem outputReportItem)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.MatchLevelSummaryReport(parameters, null, ReportOutput, outputReportItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.MatchLevelSummaryReport(parameters, null, ReportOutput, outputReportItem);
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Calculates the amount of overlap between two or more trade areas.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated <c>esriReportFormat</c> parameter is removed from this proxy-method.
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="imageOptions">Configuration options for rendering the output map image.</param>
		/// <param name="outputOptions">Array of task output options. Options for this method include rendering
		/// output image (GetMapImage), creating the feature class for subsequent analysis (GetFeatureClass),
		/// and creating a report (GetReport).</param>
		/// <param name="outputAnalysisItem">Optional output repository item to save the analysis.</param>
		/// <param name="outputReportItem">Optional output repository item to save the report.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput MeasureCannibalization(MeasureCannibalizationParameters parameters, RenderingParameters imageOptions,
			TaskOutputType[] outputOptions, esriFolderItem outputAnalysisItem, esriFolderItem outputReportItem)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.MeasureCannibalization(parameters, null, imageOptions, outputOptions,
					outputAnalysisItem, outputReportItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.MeasureCannibalization(parameters, null, imageOptions, outputOptions,
				   outputAnalysisItem, outputReportItem);
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Creates a report on distribution of points within polygons.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated <c>esriReportFormat</c> parameter is removed from this proxy-method.
		/// The signature of this proxy-method is also simplified by removing the <c>OutputType[]</c> parameter
		/// which always contains the <c>OutputType.GetReport</c> option only.
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="outputReportItem">Optional output repository item to save the report.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput PointsInPolygonReport(PointsInPolygonReportParameters parameters,
			esriFolderItem outputReportItem)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.PointsInPolygonReport(parameters, null, ReportOutput, outputReportItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.PointsInPolygonReport(parameters, null, ReportOutput, outputReportItem);
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Aggregates data from a point layer (e.g. customer layer) to a polygon layer (e.g. trade area).
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated <c>esriReportFormat</c> parameter is removed from this proxy-method.
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="imageOptions">Configuration options for rendering the output map image.</param>
		/// <param name="outputOptions">Array of task output options. Options for this method include rendering
		/// output image (GetMapImage), creating the feature class for subsequent analysis (GetFeatureClass),
		/// and creating a report (GetReport).</param>
		/// <param name="outputAnalysisItem">Optional output repository item to save the analysis.</param>
		/// <param name="outputReportItem">Optional output repository item to save the report.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput SummarizePoints(SummarizePointsParameters parameters, RenderingParameters imageOptions,
			TaskOutputType[] outputOptions, esriFolderItem outputAnalysisItem, esriFolderItem outputReportItem)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.SummarizePoints(parameters, null, imageOptions, outputOptions,
					outputAnalysisItem, outputReportItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.SummarizePoints(parameters, null, imageOptions, outputOptions,
				   outputAnalysisItem, outputReportItem);
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Generates summary reports for trade areas.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated <c>esriReportFormat</c> parameter is removed from this proxy-method.
		/// The signature of this proxy-method is also simplified by removing the <c>OutputType[]</c> parameter
		/// which always contains the <c>OutputType.GetReport</c> option only.
		/// <para>Since version 10.0, an <c>outputReportItem</c> parameter is replaced with an array of output
		/// report items. The number of items in this array should coinside with the number of reports
		/// to be generated.</para>
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="outputReportItems">Optional array of output repository items to save reports.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput SummaryReports(SummaryReportsParameters parameters,
			params esriFolderItem[] outputReportItems)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.SummaryReports(parameters, null, ReportOutput, GetReportItems(outputReportItems));
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.SummaryReports(parameters, null, ReportOutput, GetReportItems(outputReportItems));
			}
			UpdateLocalUrls(output);
			return output;
		}

		// Patches output report items for provide backward compatibility.
		private esriFolderItem[] GetReportItems(esriFolderItem[] outputReportItems)
		{
			return outputReportItems == null || outputReportItems.Length == 0
				|| outputReportItems.Length == 1 && outputReportItems[0] == null ?
				null : outputReportItems;
		}

		/// <summary>
		/// Creates a report which includes map image(s) accompanied with custom user data and graphics.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated <c>esriReportFormat</c> parameter is removed from this proxy-method.
		/// The signature of this proxy-method is also simplified by removing the <c>OutputType[]</c> parameter
		/// which always contains the <c>OutputType.GetReport</c> option only.
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for the report.</param>
		/// <param name="outputReportItem">Optional output repository item to save the report.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput MapReport(MapReportParameters parameters, esriFolderItem outputReportItem)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.MapReport(parameters, null, ReportOutput, outputReportItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.MapReport(parameters, null, ReportOutput, outputReportItem);
			}
			UpdateLocalUrls(output);
			return output;
		}

		#endregion

		#region Proxy-methods for Segmentation Tasks

		/// <summary>
		/// Displays core and developmental segments in a table with customer count and percent, market area count and percent, and index.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated <c>esriReportFormat</c> parameter is removed from this proxy-method.
		/// The signature of this proxy-method is also simplified by removing the <c>OutputType[]</c> parameter
		/// which always contains the <c>OutputType.GetReport</c> option only.
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="outputReportItem">Optional output repository item to save the report.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput CoreDevelopSegments(CoreDevelopSegmentsParameters parameters,
			esriFolderItem outputReportItem)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.CoreDevelopSegments(parameters, ReportOutput, null, outputReportItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.CoreDevelopSegments(parameters, ReportOutput, null, outputReportItem);
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Divides Tapestry segments into one of four categories.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated <c>esriReportFormat</c> parameter is removed from this proxy-method.
		/// The signature of this proxy-method is also simplified by removing the <c>OutputType[]</c> parameter
		/// which always contains the <c>OutputType.GetReport</c> option only.
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="outputReportItem">Optional output repository item to save the report.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput GamePlanChart(GamePlanChartParameters parameters,
			esriFolderItem outputReportItem)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.GamePlanChart(parameters, null, ReportOutput, outputReportItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.GamePlanChart(parameters, null, ReportOutput, outputReportItem);
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Divides sites in the area of interest into one of four categories and thematically shades each site based on the output from the Game Plan Chart.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated <c>esriReportFormat</c> parameter is removed from this proxy-method.
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="imageOptions">Configuration options for rendering the output map image.</param>
		/// <param name="outputOptions">Array of task output options. Options for this method include rendering
		/// output image (GetMapImage), creating the feature class for subsequent analysis (GetFeatureClass),
		/// and creating a report (GetReport).</param>
		/// <param name="outputAnalysisItem">Optional output repository item to save the analysis.</param>
		/// <param name="outputReportItem">Optional output repository item to save the report.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput GamePlanMap(GamePlanMapParameters parameters, RenderingParameters imageOptions,
			TaskOutputType[] outputOptions, esriFolderItem outputAnalysisItem, esriFolderItem outputReportItem)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.GamePlanMap(parameters, null, imageOptions, outputOptions,
					outputAnalysisItem, outputReportItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.GamePlanMap(parameters, null, imageOptions, outputOptions,
				   outputAnalysisItem, outputReportItem);
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Compares up to three target profiles to a single base profile.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated <c>esriReportFormat</c> parameter is removed from this proxy-method.
		/// The signature of this proxy-method is also simplified by removing the <c>OutputType[]</c> parameter
		/// which always contains the <c>OutputType.GetReport</c> option only.
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="outputReportItem">Optional output repository item to save the report.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput MultiProfileBarChart(MultiProfileBarChartParameters parameters,
			esriFolderItem outputReportItem)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.MultiProfileBarChart(parameters, null, ReportOutput, outputReportItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.MultiProfileBarChart(parameters, null, ReportOutput, outputReportItem);
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Creates a new Tapestry Segmentation Profile on the given polygon layer.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated output report item parameter and <c>OutputType[]</c> parameter
		/// are removed from this proxy-method.
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="outputProfileItem">Optional output repository item to save the Segmentation Profile.</param>
		/// <returns>Task result output.</returns>
		public string ProfileByAreaSummation(ProfileByAreaSummationParameters parameters,
			esriFolderItem outputProfileItem)
		{
			SetAnalysisParameters(parameters);

			try
			{
				return Client.ProfileByAreaSummation(parameters, null, outputProfileItem, null);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.ProfileByAreaSummation(parameters, null, outputProfileItem, null);
			}
		}

		/// <summary>
		/// Creates a new Tapestry Segmentation Profile from a table of aggregated segmentation information.
		/// </summary>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="outputProfileItem">Optional output repository item to save the Segmentation Profile.</param>
		/// <returns>Task result output.</returns>
		public string ProfileByImportFromTable(ProfileByImportFromTableParameters parameters,
			esriFolderItem outputProfileItem)
		{
			SetAnalysisParameters(parameters);

			try
			{
				return Client.ProfileByImportFromTable(parameters, outputProfileItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.ProfileByImportFromTable(parameters, outputProfileItem);
			}
		}

		/// <summary>
		/// Creates a new Tapestry Segmentation Profile from a table of address records.
		/// </summary>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="outputProfileItem">Optional output repository item to save the Segmentation Profile.</param>
		/// <returns>Task result output.</returns>
		public string ProfileByTableGeocoding(ProfileByTableGeocodingParameters parameters,
			esriFolderItem outputProfileItem)
		{
			SetAnalysisParameters(parameters);

			try
			{
				return Client.ProfileByTableGeocoding(parameters, outputProfileItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.ProfileByTableGeocoding(parameters, outputProfileItem);
			}
		}

		/// <summary>
		/// Compares a target profile to a base profile.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated <c>esriReportFormat</c> parameter is removed from this proxy-method.
		/// The signature of this proxy-method is also simplified by removing the <c>OutputType[]</c> parameter
		/// which always contains the <c>OutputType.GetReport</c> option only.
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="outputReportItem">Optional output repository item to save the report.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput ProfileComparisonChart(ProfileComparisonChartParameters parameters,
			esriFolderItem outputReportItem)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.ProfileComparisonChart(parameters, null, ReportOutput, outputReportItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.ProfileComparisonChart(parameters, null, ReportOutput, outputReportItem);
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Creates Tapestry Segmentation Report from a Segmentation Profile.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated <c>esriReportFormat</c> parameter is removed from this proxy-method.
		/// The signature of this proxy-method is also simplified by removing the <c>OutputType[]</c> parameter
		/// which always contains the <c>OutputType.GetReport</c> option only.
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="outputReportItem">Optional output repository item to save the report.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput ReportFromProfile(ReportFromProfileParameters parameters,
			esriFolderItem outputReportItem)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.ReportFromProfile(parameters, null, ReportOutput, outputReportItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.ReportFromProfile(parameters, null, ReportOutput, outputReportItem);
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Creates the Tapestry Profile Report based on comparison of your customer target Segmentation Profile
		/// to a base Segmentation Profile.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated <c>esriReportFormat</c> parameter is removed from this proxy-method.
		/// The signature of this proxy-method is also simplified by removing the <c>OutputType[]</c> parameter
		/// which always contains the <c>OutputType.GetReport</c> option only.
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="outputReportItem">Optional output repository item to save the report.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput TapestryProfileReport(TapestryProfileReportParameters parameters,
			esriFolderItem outputReportItem)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.TapestryProfileReport(parameters, null, ReportOutput, outputReportItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.TapestryProfileReport(parameters, null, ReportOutput, outputReportItem);
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Composes a Tapestry segmentation target group by an array of targets.
		/// </summary>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="outputTargetGroupItem">Optional output repository item to save the target group.</param>
		/// <returns>Task result output.</returns>
		public string TargetGroup(TargetGroupParameters parameters,
			esriFolderItem outputTargetGroupItem)
		{
			SetAnalysisParameters(parameters);

			try
			{
				return Client.TargetGroup(parameters, outputTargetGroupItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.TargetGroup(parameters, outputTargetGroupItem);
			}
		}

		/// <summary>
		/// Thematically shades selected Tapestry segments in a single color.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated <c>esriReportFormat</c> parameter is removed from this proxy-method.
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="imageOptions">Configuration options for rendering the output map image.</param>
		/// <param name="outputOptions">Array of task output options. Options for this method include rendering
		/// output image (GetMapImage), creating the feature class for subsequent analysis (GetFeatureClass),
		/// and creating a report (GetReport).</param>
		/// <param name="outputAnalysisItem">Optional output repository item to save the analysis.</param>
		/// <param name="outputReportItem">Optional output repository item to save the report.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput TargetMap(TargetMapParameters parameters, RenderingParameters imageOptions,
			TaskOutputType[] outputOptions, esriFolderItem outputAnalysisItem, esriFolderItem outputReportItem)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.TargetMap(parameters, null, imageOptions, outputOptions,
					outputAnalysisItem, outputReportItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.TargetMap(parameters, null, imageOptions, outputOptions,
				   outputAnalysisItem, outputReportItem);
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Thematically shades the percent penetration of selected segments.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated <c>esriReportFormat</c> parameter is removed from this proxy-method.
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="imageOptions">Configuration options for rendering the output map image.</param>
		/// <param name="outputOptions">Array of task output options. Options for this method include rendering
		/// output image (GetMapImage), creating the feature class for subsequent analysis (GetFeatureClass),
		/// and creating a report (GetReport).</param>
		/// <param name="outputAnalysisItem">Optional output repository item to save the analysis.</param>
		/// <param name="outputReportItem">Optional output repository item to save the report.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput TargetPenetrationMap(TargetPenetrationMapParameters parameters, RenderingParameters imageOptions,
			TaskOutputType[] outputOptions, esriFolderItem outputAnalysisItem, esriFolderItem outputReportItem)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.TargetPenetrationMap(parameters, null, imageOptions, outputOptions,
					outputAnalysisItem, outputReportItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.TargetPenetrationMap(parameters, null, imageOptions, outputOptions,
				   outputAnalysisItem, outputReportItem);
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Creates a report to compare the Tapestry profile of your core and developmental customers to the
		/// Tapestry profile of a syndicated survey.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated <c>esriReportFormat</c> parameter is removed from this proxy-method.
		/// The signature of this proxy-method is also simplified by removing the <c>OutputType[]</c> parameter
		/// which always contains the <c>OutputType.GetReport</c> option only.
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="outputReportItem">Optional output repository item to save the report.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput UnderstandingTargetCustomers(UnderstandingTargetCustomersParameters parameters,
			esriFolderItem outputReportItem)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.UnderstandingTargetCustomers(parameters, null, ReportOutput, outputReportItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.UnderstandingTargetCustomers(parameters, null, ReportOutput, outputReportItem);
			}
			UpdateLocalUrls(output);
			return output;
		}

		#endregion

		#region Proxy-methods for Trade Area Tasks

		/// <summary>
		/// Creates trade areas around store features based on percentages of their customers measured in straight-line distance from stores.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated <c>esriReportFormat</c> parameter is removed from this proxy-method.
		/// <para>Since version 10.0, an <c>outputReportItem</c> parameter is replaced with an array of output
		/// report items. The number of items in this array should coincide with the number of reports
		/// generated.</para>
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="imageOptions">Configuration options for rendering the output map image.</param>
		/// <param name="outputOptions">Array of task output options. Options for this method include rendering
		/// output image (GetMapImage), creating the feature class for subsequent analysis (GetFeatureClass),
		/// and creating a report (GetReport).</param>
		/// <param name="outputAnalysisItem">Optional output repository item to save the analysis.</param>
		/// <param name="outputReportItems">Optional array of output repository items to save reports.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput CustomerDerivedAreas(CustomerDerivedAreasParameters parameters, RenderingParameters imageOptions,
			TaskOutputType[] outputOptions, esriFolderItem outputAnalysisItem, params esriFolderItem[] outputReportItems)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.CustomerDerivedAreas(parameters, null, imageOptions, outputOptions,
					outputAnalysisItem, GetReportItems(outputReportItems));
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.CustomerDerivedAreas(parameters, null, imageOptions, outputOptions,
				   outputAnalysisItem, GetReportItems(outputReportItems));
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Creates trade areas based on the driving time or driving distance around store features.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated <c>esriReportFormat</c> parameter is removed from this proxy-method.
		/// <para>Since version 10.0, an <c>outputReportItem</c> parameter is replaced with an array of output
		/// report items. The number of items in this array should coincide with the number of reports
		/// generated.</para>
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="imageOptions">Configuration options for rendering the output map image.</param>
		/// <param name="outputOptions">Array of task output options. Options for this method include rendering
		/// output image (GetMapImage), creating the feature class for subsequent analysis (GetFeatureClass),
		/// and creating a report (GetReport).</param>
		/// <param name="outputAnalysisItem">Optional output repository item to save the analysis.</param>
		/// <param name="outputReportItems">Optional array of output repository items to save reports.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput DriveTime(DriveTimeParameters parameters, RenderingParameters imageOptions,
			TaskOutputType[] outputOptions, esriFolderItem outputAnalysisItem, params esriFolderItem[] outputReportItems)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.DriveTime(parameters, null, imageOptions, outputOptions,
					outputAnalysisItem, GetReportItems(outputReportItems));
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.DriveTime(parameters, null, imageOptions, outputOptions,
				   outputAnalysisItem, GetReportItems(outputReportItems));
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Generates an equidistant vector-based grid network for a specified area.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated <c>esriReportFormat</c> parameter is removed from this proxy-method.
		/// <para>Since version 10.0, an <c>outputReportItem</c> parameter is replaced with an array of output
		/// report items. The number of items in this array should coincide with the number of reports
		/// generated.</para>
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="imageOptions">Configuration options for rendering the output map image.</param>
		/// <param name="outputOptions">Array of task output options. Options for this method include rendering
		/// output image (GetMapImage), creating the feature class for subsequent analysis (GetFeatureClass),
		/// and creating a report (GetReport).</param>
		/// <param name="outputAnalysisItem">Optional output repository item to save the analysis.</param>
		/// <param name="outputReportItems">Optional array of output repository items to save reports.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput Grids(GridsParameters parameters, RenderingParameters imageOptions,
			TaskOutputType[] outputOptions, esriFolderItem outputAnalysisItem, params esriFolderItem[] outputReportItems)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.Grids(parameters, null, imageOptions, outputOptions,
					outputAnalysisItem, GetReportItems(outputReportItems));
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.Grids(parameters, null, imageOptions, outputOptions,
				   outputAnalysisItem, GetReportItems(outputReportItems));
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Removes overlap (cannibalization) between trade areas.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated <c>esriReportFormat</c> parameter is removed from this proxy-method.
		/// <para>Since version 10.0, an <c>outputReportItem</c> parameter is replaced with an array of output
		/// report items. The number of items in this array should coincide with the number of reports
		/// generated.</para>
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="imageOptions">Configuration options for rendering the output map image.</param>
		/// <param name="outputOptions">Array of task output options. Options for this method include rendering
		/// output image (GetMapImage), creating the feature class for subsequent analysis (GetFeatureClass),
		/// and creating a report (GetReport).</param>
		/// <param name="outputAnalysisItem">Optional output repository item to save the analysis.</param>
		/// <param name="outputReportItems">Optional array of output repository items to save reports.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput RemoveOverlap(RemoveOverlapParameters parameters, RenderingParameters imageOptions,
			TaskOutputType[] outputOptions, esriFolderItem outputAnalysisItem, params esriFolderItem[] outputReportItems)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.RemoveOverlap(parameters, null, imageOptions, outputOptions,
					outputAnalysisItem, GetReportItems(outputReportItems));
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.RemoveOverlap(parameters, null, imageOptions, outputOptions,
				   outputAnalysisItem, GetReportItems(outputReportItems));
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Creates a new feature class of a ring-based trade area using a set of radii.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated <c>esriReportFormat</c> parameter is removed from this proxy-method.
		/// <para>Since version 10.0, an <c>outputReportItem</c> parameter is replaced with an array of output
		/// report items. The number of items in this array should coincide with the number of reports
		/// generated.</para>
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="imageOptions">Configuration options for rendering the output map image.</param>
		/// <param name="outputOptions">Array of task output options. Options for this method include rendering
		/// output image (GetMapImage), creating the feature class for subsequent analysis (GetFeatureClass),
		/// and creating a report (GetReport).</param>
		/// <param name="outputAnalysisItem">Optional output repository item to save the analysis.</param>
		/// <param name="outputReportItems">Optional array of output repository items to save reports.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput SimpleRings(SimpleRingsParameters parameters, RenderingParameters imageOptions,
			TaskOutputType[] outputOptions, esriFolderItem outputAnalysisItem, params esriFolderItem[] outputReportItems)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.SimpleRings(parameters, null, imageOptions, outputOptions,
					outputAnalysisItem, GetReportItems(outputReportItems));
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.SimpleRings(parameters, null, imageOptions, outputOptions,
				   outputAnalysisItem, GetReportItems(outputReportItems));
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Generates trade areas based on standard geographic units.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated <c>esriReportFormat</c> parameter is removed from this proxy-method.
		/// <para>Since version 10.0, an <c>outputReportItem</c> parameter is replaced with an array of output
		/// report items. The number of items in this array should coincide with the number of reports
		/// generated.</para>
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="imageOptions">Configuration options for rendering the output map image.</param>
		/// <param name="outputOptions">Array of task output options. Options for this method include rendering
		/// output image (GetMapImage), creating the feature class for subsequent analysis (GetFeatureClass),
		/// and creating a report (GetReport).</param>
		/// <param name="outputAnalysisItem">Optional output repository item to save the analysis.</param>
		/// <param name="outputReportItems">Optional array of output repository items to save reports.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput StandardLevelsOfGeography(StandardLevelsOfGeographyParameters parameters, RenderingParameters imageOptions,
			TaskOutputType[] outputOptions, esriFolderItem outputAnalysisItem, params esriFolderItem[] outputReportItems)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.StandardLevelsOfGeography(parameters, null, imageOptions, outputOptions,
					outputAnalysisItem, GetReportItems(outputReportItems));
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.StandardLevelsOfGeography(parameters, null, imageOptions, outputOptions,
				   outputAnalysisItem, GetReportItems(outputReportItems));
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Creates rings around stores. The radii of the rings are determined by expanding out from
		/// the store location until they meet the given criteria.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// Deprecated <c>esriReportFormat</c> parameter is removed from this proxy-method.
		/// <para>Since version 10.0, an <c>outputReportItem</c> parameter is replaced with an array of output
		/// report items. The number of items in this array should coincide with the number of reports
		/// generated.</para>
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="imageOptions">Configuration options for rendering the output map image.</param>
		/// <param name="outputOptions">Array of task output options. Options for this method include rendering
		/// output image (GetMapImage), creating the feature class for subsequent analysis (GetFeatureClass),
		/// and creating a report (GetReport).</param>
		/// <param name="outputAnalysisItem">Optional output repository item to save the analysis.</param>
		/// <param name="outputReportItems">Optional array of output repository items to save reports.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput ThresholdRings(ThresholdRingsParams parameters, RenderingParameters imageOptions,
			TaskOutputType[] outputOptions, esriFolderItem outputAnalysisItem, params esriFolderItem[] outputReportItems)
		{
			SetAnalysisParameters(parameters);
			TaskResultOutput output;

			try
			{
				output = Client.ThresholdRings(parameters, null, imageOptions, outputOptions,
					outputAnalysisItem, GetReportItems(outputReportItems));
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.ThresholdRings(parameters, null, imageOptions, outputOptions,
				   outputAnalysisItem, GetReportItems(outputReportItems));
			}
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Calculates radii of threshold rings.
		/// </summary>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <returns>Threshold rings radii.</returns>
		public ThresholdRingRadii[] ThresholdRingsRadii(ThresholdRingsRadiiParams parameters)
		{
			SetAnalysisParameters(parameters);

			try
			{
				return Client.ThresholdRingsRadii(parameters);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.ThresholdRingsRadii(parameters);
			}
		}

		#endregion

		#region Proxy-methods for Businesses Search Utilitues

		/// <summary>
		/// Returns business points matching the given search criteria.
		/// </summary>
		/// <param name="parameters">Configuration options for the analysis.</param>
		/// <returns>Task result output whose <c>RecordSet</c> parameter contains a feature record set with selected business points.</returns>
		public TaskResultOutput SelectBusinesses(SelectBusinessesParameters parameters)
		{
			SetAnalysisParameters(parameters);

			try
			{
				return Client.SelectBusinesses(parameters);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.SelectBusinesses(parameters);
			}
		}

		/// <summary>
		/// Returns business locations matching the given search criteria.
		/// </summary>
		/// <param name="parameters">Configuration options for the analysis.</param>
		/// <returns>Task result output whose <c>RecordSet</c> parameter contains a table of business locations selected.</returns>
		public TaskResultOutput SelectLocations(SelectLocationsParameters parameters)
		{
			SetAnalysisParameters(parameters);

			try
			{
				return Client.SelectLocations(parameters);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.SelectLocations(parameters);
			}
		}

		#endregion

		#region Proxy-methods for Lookup Utilities

		/// <summary>
		/// Gets the list of all available report formats.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// The active dataset ID is requested from properties of this instance.
		/// </note>
		/// </remarks>
		/// <returns>Array containing all available format names.</returns>
		public string[] GetAvailableReportFormats()
		{
			try
			{
				return Client.GetAvailableReportFormats(ActiveDatasetID);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.GetAvailableReportFormats(ActiveDatasetID);
			}
		}

		/// <summary>
		/// Gets IDs and names of all available custom data layers.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// The active dataset ID is requested from properties of this instance.
		/// </note>
		/// </remarks>
		/// <returns>Array containing IDs and names of all available custom data layers.</returns>
		public IDNamePair[] GetCustomLayers()
		{
			try
			{
				return Client.GetCustomLayers(ActiveDatasetID);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.GetCustomLayers(ActiveDatasetID);
			}
		}

		/// <summary>
		/// Gets the list of all available hierarchies of data layers.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// The active dataset ID is requested from properties of this instance.
		/// </note>
		/// </remarks>
		/// <returns>Array containing available hierarchies of data layers.</returns>
		public string[] GetDataHierarchies()
		{
			try
			{
				return Client.GetDataHierarchies(ActiveDatasetID);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.GetDataHierarchies(ActiveDatasetID);
			}
		}

		/// <summary>
		/// Gets the list of IDs of all available datasets.
		/// </summary>
		/// <returns>Array containing all available datasets.</returns>
		public string[] GetDatasets()
		{
			try
			{
				return Client.GetDatasets();
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.GetDatasets();
			}
		}

		/// <summary>
		/// Gets the default spatial reference.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// The active dataset ID is requested from properties of this instance.
		/// </note>
		/// </remarks>
		/// <returns>The default spatial reference.</returns>
		public SpatialReference GetDefaultSpatialReference()
		{
			try
			{
				return Client.GetDefaultSpatialReference(ActiveDatasetID);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.GetDefaultSpatialReference(ActiveDatasetID);
			}
		}

		/// <summary>
		/// Gets the list of address locator fields.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// The active dataset ID is requested from properties of this instance.
		/// </note>
		/// </remarks>
		/// <returns>Array containing names of address locator fields.</returns>
		public string[] GetLocatorFields()
		{
			try
			{
				return Client.GetLocatorFields(ActiveDatasetID);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.GetLocatorFields(ActiveDatasetID);
			}
		}

		/// <summary>
		/// Gets the list of all available MRI groups.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// The active dataset ID is requested from properties of this instance.
		/// </note>
		/// </remarks>
		/// <returns>Array containing names of all available MRI groups.</returns>
		public string[] GetMRIGroups()
		{
			try
			{
				return Client.GetMRIGroups(ActiveDatasetID);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.GetMRIGroups(ActiveDatasetID);
			}
		}

		/// <summary>
		/// Gets the list of all available report templates for summary reports.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// The active dataset ID is requested from properties of this instance.
		/// </note>
		/// </remarks>
		/// <returns>Array containing all available report templates.</returns>
		public ReportTemplateInfo[] GetReportTemplates()
		{
			try
			{
				return Client.GetReportTemplates(ActiveDatasetID);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.GetReportTemplates(ActiveDatasetID);
			}
		}

		/// <summary>
		/// Gets the list of all available segmentation bases.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// The active dataset ID is requested from properties of this instance.
		/// </note>
		/// </remarks>
		/// <returns>Array containing names of all available segmentation bases.</returns>
		public string[] GetSegBases()
		{
			try
			{
				return Client.GetSegBases(ActiveDatasetID);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.GetSegBases(ActiveDatasetID);
			}
		}

		/// <summary>
		/// Gets the list of all available segmentation levels.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// The active dataset ID is requested from properties of this instance.
		/// </note>
		/// </remarks>
		/// <returns>IDs and names of available segmentation levels.</returns>
		public IDNamePair[] GetSegmentationLevels()
		{
			try
			{
				return Client.GetSegmentationLevels(ActiveDatasetID);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.GetSegmentationLevels(ActiveDatasetID);
			}
		}

		/// <summary>
		/// Gets the list of all available segments.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// The active dataset ID is requested from properties of this instance.
		/// </note>
		/// </remarks>
		/// <returns>IDs and names of available segmentations.</returns>
		public SegmentInfo[] GetSegments()
		{
			try
			{
				return Client.GetSegments(ActiveDatasetID);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.GetSegments(ActiveDatasetID);
			}
		}

		/// <summary>
		/// Gets standard geography levels available in the Business Analyst Server core dataset.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// The active dataset ID is requested from properties of this instance.
		/// </note>
		/// </remarks>
		/// <returns>IDs and names of available geography levels.</returns>
		public IDNamePair[] GetStandardGeographyLevels()
		{
			try
			{
				return Client.GetStandardGeographyLevels(ActiveDatasetID);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.GetStandardGeographyLevels(ActiveDatasetID);
			}
		}

		/// <overloads>Gets the list of summarization variables.</overloads>
		/// <summary>
		/// Gets the list of all summarization variables available for the default data hierarchy.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// The active dataset ID is requested from properties of this instance.
		/// </note>
		/// </remarks>
		/// <returns>Array containing all summarization variables.</returns>
		public SummarizationInfo[] GetSummarizations()
		{
			return GetSummarizations(null);
		}

		/// <summary>
		/// Gets the list of all summarization variables available for the given data hierarchy.
		/// </summary>
		/// <remarks>
		/// If the given data hierarchy is not found, the Business Analyst Server tries to find a data layer
		/// with the given ID for compatibility with version 9.3.1 SP1. If this parameter is null,
		/// the default data hierarchy is used in the request.
		/// <note type="caution">
		/// The active dataset ID is requested from properties of this instance.
		/// </note>
		/// </remarks>
		/// <param name="dataHierarchy">Name of a hierarchy of data layers which summarization variables are requested. </param>
		/// <returns>Array containing all summarization variables.</returns>
		public SummarizationInfo[] GetSummarizations(string dataHierarchy)
		{
			return GetSummarizations(dataHierarchy, null, null, null, null);
		}

		/// <summary>
		/// Gets the list of summarization variables available for the given data hierarchy.
		/// </summary>
		/// <remarks>
		/// If the given data hierarchy is not found, the Business Analyst Server tries to find a data layer
		/// with the given ID for compatibility with version 9.3.1 SP1. If this parameter is null,
		/// the default data hierarchy is used in the request.
		/// <note type="caution">
		/// The active dataset ID is requested from properties of this instance.
		/// </note>
		/// </remarks>
		/// <param name="dataHierarchy">Name of a hierarchy of data layers which summarization variables are requested. </param>
		/// <param name="visibility">Array of requested attributes or attribute groups of summarization variables.</param>
		/// <param name="query">A query to filter summarizations.</param>
		/// <param name="start">Zero-based index of the first summarization variable to be requested.</param>
		/// <param name="count">The maximum number of summarizations requested.</param>
		/// <returns>Array containing required summarization variables.</returns>
		public SummarizationInfo[] GetSummarizations(string dataHierarchy, string[] visibility,
			string query, int? start, int? count)
		{
			try
			{
				return Client.GetSummarizations(ActiveDatasetID, dataHierarchy, visibility, query, start, count);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.GetSummarizations(ActiveDatasetID, dataHierarchy, visibility, query, start, count);
			}
		}

		/// <summary>
		/// Gets the version of Business Analyst Server.
		/// </summary>
		/// <returns>The Business Analyst Server version.</returns>
		public string GetVersion()
		{
			try
			{
				return Client.GetVersion();
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.GetVersion();
			}
		}

		#endregion

		#region Proxy-methods for Standard Geographies Utilities

		/// <summary>
		/// Creates a study/trade area consisting of standard geography regions.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// The signature of this proxy-method is simplified by removing the <c>OutputType[]</c> parameter
		/// which always contains the <c>OutputType.FeatureClass</c> option only.
		/// </note>
		/// </remarks>
		/// <param name="parameters">Configuration options for analysis.</param>
		/// <param name="outputAnalysisItem">Optional output repository item to save the analysis.</param>
		/// <returns>Task result output.</returns>
		public TaskResultOutput CreateStdGeographies(CreateStdGeographiesParams parameters,
			esriFolderItem outputAnalysisItem)
		{
			SetAnalysisParameters(parameters);
			TaskOutputType[] outputOptions = { TaskOutputType.GetFeatureClass };

			try
			{
				return Client.CreateStdGeographies(parameters, outputOptions, outputAnalysisItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.CreateStdGeographies(parameters, outputOptions, outputAnalysisItem);
			}
		}

		/// <summary>
		/// Returns IDs and names of features from specified geography levels that match the given location.
		/// </summary>
		/// <param name="parameters">Geographic search parameters.</param>
		/// <returns>IDs and names of features of geography levels matching the given location.</returns>
		public StdGeographiesReturnedFeature[] ReturnStdGeography(ReturnStdGeographyParams parameters)
		{
			SetAnalysisParameters(parameters);

			try
			{
				return Client.ReturnStdGeography(parameters);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.ReturnStdGeography(parameters);
			}
		}

		/// <summary>
		/// Gets geography location names and IDs matching a string-based query.
		/// </summary>
		/// <param name="parameters">Geographic search parameters.</param>
		/// <returns>IDs and names of features.</returns>
		public StdGeographiesReturnedFeature[] StdGeographiesByAttributes(StdGeographiesByAttributesParams parameters)
		{
			SetAnalysisParameters(parameters);

			try
			{
				return Client.StdGeographiesByAttributes(parameters);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.StdGeographiesByAttributes(parameters);
			}
		}

		/// <summary>
		/// Gets geography location names and IDs matching a given extent.
		/// </summary>
		/// <param name="parameters">Geographic search parameters.</param>
		/// <returns>IDs and names of features.</returns>
		public StdGeographiesReturnedFeature[] StdGeographiesFromExtent(StdGeographiesFromExtentParams parameters)
		{
			SetAnalysisParameters(parameters);

			try
			{
				return Client.StdGeographiesFromExtent(parameters);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.StdGeographiesFromExtent(parameters);
			}
		}

		/// <summary>
		/// Gets geography location names and IDs matching a query to the hierarchy of standard geography levels.
		/// </summary>
		/// <param name="parameters">Geographic search parameters.</param>
		/// <returns>IDs and names of features.</returns>
		public StdGeographiesReturnedFeature[] StdGeographiesFromTree(StdGeographiesFromTreeParams parameters)
		{
			SetAnalysisParameters(parameters);

			try
			{
				return Client.StdGeographiesFromTree(parameters);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.StdGeographiesFromTree(parameters);
			}
		}

		/// <summary>
		/// Returns IDs and names of geography levels which contain features matching the given search criterion.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// This proxy-method changes the type of returned array from <c>StdGeographiesReturnedFeature[]</c>
		/// to <c>IDNamePair[]</c>.
		/// </note>
		/// </remarks>
		/// <param name="parameters">Geographic search parameters.</param>
		/// <returns>IDs and names of geography levels.</returns>
		public IDNamePair[] StdGeographiesList(StdGeographiesListParams parameters)
		{
			SetAnalysisParameters(parameters);
			StdGeographiesReturnedFeature[] result;

			try
			{
				result = Client.StdGeographiesList(parameters);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				result = Client.StdGeographiesList(parameters);
			}

			// Fix the bug with the wrong type of the returned array.
			IDNamePair[] levels = new IDNamePair[result.Length];
			for (int i = 0; i < levels.Length; i++)
			{
				IDNamePair level = new IDNamePair();
				level.ID = result[i].FeatureID;
				level.Name = result[i].FeatureName;
				levels[i] = level;
			}
			return levels;
		}

		/// <summary>
		/// Returns the tree of standard geography levels.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// The active dataset ID is requested from properties of this instance.
		/// </note>
		/// </remarks>
		/// <returns>XML serialized tree of standard geography levels.</returns>
		public string StdGeographiesTree()
		{
			StdGeographiesTreeParams parameters = new StdGeographiesTreeParams();
			SetAnalysisParameters(parameters);

			try
			{
				return Client.StdGeographiesTree(parameters);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.StdGeographiesTree(parameters);
			}
		}

		#endregion

		#region Proxy-methods for Rendering Utilites

		/// <summary>
		/// Adds layers to the default map.
		/// </summary>
		/// <param name="layerItems">Layer items in the repository that will be added to the default map.</param>
		public void AddLayersToMap(esriLayerItem[] layerItems)
		{
			AddLayersToMapParameters parameters = new AddLayersToMapParameters();
			parameters.LayerItems = layerItems;
			parameters.IsFullErrorMessage = isFullErrorMessage;

			try
			{
				Client.AddLayersToMap(parameters);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				Client.AddLayersToMap(parameters);
			}
		}

		/// <summary>
		/// Removes layers from the default map.
		/// </summary>
		/// <param name="layerNames">Names of layers to be removed from the default map.</param>
		/// <returns>True value if at least one layer was removed.</returns>
		public bool RemoveLayersFromMap(string[] layerNames)
		{
			RemoveLayersFromMapParameters parameters = new RemoveLayersFromMapParameters();
			parameters.LayerNames = layerNames;
			parameters.IsFullErrorMessage = isFullErrorMessage;

			try
			{
				return Client.RemoveLayersFromMap(parameters);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.RemoveLayersFromMap(parameters);
			}
		}

		/// <summary>
		/// Gets the default map description.
		/// </summary>
		/// <returns>Map description object.</returns>
		public MapDescription GetDefaultMapDescription()
		{
			try
			{
				return Client.GetDefaultMapDescription();
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.GetDefaultMapDescription();
			}
		}

		/// <summary>
		/// Renders repository items and returns a map image.
		/// </summary>
		/// <remarks>If <paramref name="MapDescription"/> is null, items are rendered on the default map.
		/// If <paramref name="ImageDescription"/> is null, items are rendered with default image configuration options. 
		/// </remarks>
		/// <param name="analysisItems">Layer items in the repository that will be rendered as an image.</param>
		/// <param name="imageDescription">Image configuration options for rendering output.</param>
		/// <param name="mapDescription">Map configuration options for rendering output.</param>
		/// <returns>Map image.</returns>
		public MapImage RenderAnalyses(esriLayerItem[] analysisItems, ImageDescription imageDescription,
			MapDescription mapDescription)
		{
			MapImage result;

			try
			{
				result = Client.RenderAnalyses(analysisItems, imageDescription, mapDescription);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				result = Client.RenderAnalyses(analysisItems, imageDescription, mapDescription);
			}

			result.ImageURL = UpdateLocalUrl(result.ImageURL);
			return result;
		}

		#endregion

		#region Proxy-methods for Repository Utilites

		/// <summary>
		/// Adds features to a repository feature layer.
		/// </summary>
		/// <remarks>
		/// <para>
		/// This method allows the user to add features to a feature layer having a point or polygon geometry.
		/// The <c>sourceLayer</c> and <c>destinationItem</c> layers should have the same type of geometry
		/// and the same spatial reference.
		/// </para>
		/// <para>
		/// While adding features to the destination layer, alien attributes of features are ignored.
		/// If the destination layer contains attributes that are missing in the source layer, its values
		/// for added features are initialized to default values.
		/// </para>
		/// <para>
		/// When added to the destination layer, features of the source layer receive serial values for
		/// the Object ID attribute starting after the last feature Object ID in the destination layer.
		/// Because the Object ID attribute value is automatically assigned to new features of the destination
		/// layer, it is not necessary to specify the Object ID attribute for features of the source feature layer.
		/// If this attribute is specified, it is ignored.
		/// </para>
		/// </remarks>
		/// <param name="sourceLayer">Source data layer to get features. The source layer should be
		/// specified by the <c>RecordSet</c> or <c>Item</c> or <c>Points</c> property value.</param>
		/// <param name="destinationItem">Folder item containing a feature layer to be updated.</param>
		public void AddFeatures(PointLayer sourceLayer, esriFolderItem destinationItem)
		{
			AddFeaturesParameters parameters = new AddFeaturesParameters();
			parameters.SourceLayer = sourceLayer;
			parameters.DestinationItem = destinationItem;
			parameters.IsFullErrorMessage = isFullErrorMessage;

			try
			{
				Client.AddFeatures(parameters);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				Client.AddFeatures(parameters);
			}
		}

		/// <overloads>Creates a copy of the project under another name.</overloads>
		/// <summary>
		/// Creates a copy of the project under another name in the same workspace.
		/// </summary>
		/// <remarks>If the source project is absent, this method throws an exception.
		/// If the target project already exists, this method has no effect.</remarks>
		/// <param name="workspaceName">Workspace name.</param>
		/// <param name="projectName">Project name.</param>
		/// <param name="newProjectName">New project name.</param>
		public void CopyProject(string workspaceName, string projectName, string newProjectName)
		{
			CopyProject(workspaceName, projectName, newProjectName, null);
		}

		/// <summary>
		/// Creates a copy of the project under another name in the given workspace.
		/// </summary>
		/// <remarks>If the source project is absent, this method throws an exception.
		/// If the target project already exists, this method has no effect.</remarks>
		/// <param name="workspaceName">Workspace name.</param>
		/// <param name="projectName">Project name.</param>
		/// <param name="newProjectName">New project name.</param>
		/// <param name="destinationWorkspaceName">Destination workspace name (nullable).</param>
		public void CopyProject(string workspaceName, string projectName, string newProjectName, string destinationWorkspaceName)
		{
			try
			{
				Client.CopyProject(workspaceName, projectName, newProjectName, destinationWorkspaceName);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				Client.CopyProject(workspaceName, projectName, newProjectName, destinationWorkspaceName);
			}
		}

		/// <summary>
		/// Creates a new project.
		/// </summary>
		/// <remarks>If the project to create already exists, this method throws an exception.</remarks>
		/// <param name="workspaceName">Workspace name.</param>
		/// <param name="projectName">Name of a new project.</param>
		public void CreateProject(string workspaceName, string projectName)
		{
			try
			{
				Client.CreateProject(workspaceName, projectName);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				Client.CreateProject(workspaceName, projectName);
			}
		}

		/// <summary>
		/// Creates a new workspace.
		/// </summary>
		/// <remarks>If the workspace to create already exists, this method has no effect.</remarks>
		/// <param name="workspaceName">Name of a new workspace.</param>
		public void CreateWorkspace(string workspaceName)
		{
			try
			{
				Client.CreateWorkspace(workspaceName);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				Client.CreateWorkspace(workspaceName);
			}
		}

		/// <summary>
		/// Deletes features in a repository feature layer.
		/// </summary>
		/// <param name="OIDs">Array of OIDs of features to be deleted.</param>
		/// <param name="changedItem">Folder item containing a feature layer to be modified.</param>
		/// <returns>True value if at least one feature was deleted.</returns>
		public bool DeleteFeatures(int[] OIDs, esriFolderItem changedItem)
		{
			DeleteFeaturesParameters parameters = new DeleteFeaturesParameters();
			parameters.OIDs = OIDs;
			parameters.ChangedItem = changedItem;
			parameters.IsFullErrorMessage = isFullErrorMessage;

			try
			{
				return Client.DeleteFeatures(parameters);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.DeleteFeatures(parameters);
			}
		}

		/// <overloads>Deletes an item in the repository.</overloads>
		/// <summary>
		/// Deletes an item in the repository.
		/// </summary>
		/// <remarks>If the item to delete is absent, this method has no effect.</remarks>
		/// <param name="workspaceName">Workspace name.</param>
		/// <param name="projectName">Project name.</param>
		/// <param name="folderType">Folder type of item to delete.</param>
		/// <param name="itemName">Item name.</param>
		public void DeleteItem(string workspaceName, string projectName, esriFolderType folderType, string itemName)
		{
			try
			{
				Client.DeleteItem(workspaceName, projectName, folderType, itemName);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				Client.DeleteItem(workspaceName, projectName, folderType, itemName);
			}
		}
		/// <summary>
		/// Deletes an item in the repository.
		/// </summary>
		/// <remarks>If the item to delete is absent, this method has no effect.</remarks>
		/// <param name="item">Item configuration options.</param>
		public void DeleteItem(esriFolderItem item)
		{
			DeleteItem(item.workspaceName, item.projectName, item.folderType, item.itemName);
		}

		/// <summary>
		/// Deletes a project.
		/// </summary>
		/// <remarks>If the project to delete is absent, this method throws an exception.
		/// Additionally, if the "Default Project" is deleted, all of its contents (folder items, etc.)
		/// will be deleted and this project will be recreated on the next full reload.</remarks>
		/// <param name="workspaceName">Workspace name.</param>
		/// <param name="projectName">Name of a project to delete.</param>
		public void DeleteProject(string workspaceName, string projectName)
		{
			try
			{
				Client.DeleteProject(workspaceName, projectName);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				Client.DeleteProject(workspaceName, projectName);
			}
		}

		/// <summary>
		/// Deletes a workspace.
		/// </summary>
		/// <remarks>If the workspace to delete is absent, this method has no effect.
		/// If the "Default Workspace" is deleted, all of its contents will be deleted and the workspace
		/// will be recreated on the next full reload.</remarks>
		/// <param name="workspaceName">Name of a workspace to delete.</param>
		public void DeleteWorkspace(string workspaceName)
		{
			try
			{
				Client.DeleteWorkspace(workspaceName);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				Client.DeleteWorkspace(workspaceName);
			}
		}

		/// <summary>
		/// Compresses the specified folders of a project and returns URLs for download.
		/// </summary>
		/// <remarks>This method is usually used for downloading the contents of the "Custom&#160;Data" folder.</remarks>
		/// <param name="workspaceName">Workspace name.</param>
		/// <param name="projectName">Project name.</param>
		/// <param name="folders">Array of project folders to be downloaded.</param>
		/// <returns>Array of URLs on compressed project folders.</returns>
		public string[] DownloadData(string workspaceName, string projectName, string[] folders)
		{
			string[] result;

			try
			{
				result = Client.DownloadData(workspaceName, projectName, folders);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				result = Client.DownloadData(workspaceName, projectName, folders);
			}

			// Correct local URLs
			if (domain != null)
				for (int i = 0; i < result.Length; i++)
					result[i] = UpdateLocalUrl(result[i]);

			return result;
		}

		/// <summary>
		/// Gets the list of all project items available in the given folder.
		/// </summary>
		/// <param name="workspaceName">Workspace name.</param>
		/// <param name="projectName">Project name.</param>
		/// <param name="folderType">Folder type to get list.</param>
		/// <returns>Array of items stored in the given project folder.</returns>
		public esriFolderItem[] GetListProjectItems(string workspaceName, string projectName, esriFolderType folderType)
		{
			try
			{
				return Client.GetListProjectItems(workspaceName, projectName, folderType);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.GetListProjectItems(workspaceName, projectName, folderType);
			}
		}

		/// <summary>
		/// Gets the list of all projects available in the given workspace.
		/// </summary>
		/// <param name="workspaceName">Wprkspace name.</param>
		/// <returns>Array containing names of projects available in the given workspace.</returns>
		public string[] GetProjects(string workspaceName)
		{
			try
			{
				return Client.GetProjects(workspaceName);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.GetProjects(workspaceName);
			}
		}

		/// <summary>
		/// Generates a report in the given formats from a report item previously created and stored
		/// in the repository and returns reports generated.
		/// </summary>
		/// <param name="reportItem">The report item stored in the repository.</param>
		/// <param name="reportFormats">Output formats to get the report.</param>
		/// <returns>TaskResultOutput whose Reports property contains array of ReportInfo.</returns>
		public TaskResultOutput GetReport(esriFolderItem reportItem, params string[] reportFormats)
		{
			TaskResultOutput output;

			try
			{
				output = Client.GetReport(reportItem, null, null, reportFormats);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				output = Client.GetReport(reportItem, null, null, reportFormats);
			}

			// Correct local URLs
			UpdateLocalUrls(output);
			return output;
		}

		/// <summary>
		/// Gets a unique zip-file name URL.
		/// </summary>
		/// <remarks><para>The returned URL has the following syntax:</para>
		/// <para><c>http://{Business Analyst Server host name}/arcgisoutput/_ags_T{GUID}.zip</c></para>
		/// <note type="caution">
		/// The deprecated parameter is removed from this proxy-method.
		/// </note>
		/// </remarks>
		/// <returns>A unique URL to a zip-file.</returns>
		public string GetUniqueFileName()
		{
			string result;

			try
			{
				result = Client.GetUniqueFileName(null);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				result = Client.GetUniqueFileName(null);
			}

			// Correct local URL
			return UpdateLocalUrl(result);
		}

		/// <summary>
		/// Gets the list of all available workspaces.
		/// </summary>
		/// <returns>Array containing names of all available workspaces.</returns>
		public string[] GetWorkspaces()
		{
			try
			{
				return Client.GetWorkspaces();
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.GetWorkspaces();
			}
		}

		/// <overloads>Returns a repository feature layer item as a serialized Esri Feature RecordSet.</overloads>
		/// <summary>
		/// Returns a repository feature layer item as a serialized Esri Feature RecordSet.
		/// </summary>
		/// <param name="layerItem">Folder item containing a feature layer.</param>
		/// <returns>An object containing the repository feature layer as the RecordSet property.</returns>
		public TaskResultOutput OpenFeatureLayer(esriFolderItem layerItem)
		{
			OpenFeatureLayerParameters parameters = new OpenFeatureLayerParameters();
			parameters.Item = layerItem;
			return OpenFeatureLayer(parameters);
		}

		/// <summary>
		/// Returns a repository feature layer item as a serialized Esri Feature RecordSet.
		/// </summary>
		/// <param name="parameters">Configuration options.</param>
		/// <returns>An object containing the repository feature layer as the RecordSet property.</returns>
		public TaskResultOutput OpenFeatureLayer(OpenFeatureLayerParameters parameters)
		{
			try
			{
				return Client.OpenFeatureLayer(parameters, null);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.OpenFeatureLayer(parameters, null);
			}
		}

		/// <summary>
		/// Returns XML serialized Segmentation Profile.
		/// </summary>
		/// <param name="profileItem">Folder item containing a Segmentation Profile.</param>
		/// <returns>A string containing XML serialized Segmentation Profile.</returns>
		public string OpenSegProfile(esriFolderItem profileItem)
		{
			try
			{
				return Client.OpenSegProfile(profileItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.OpenSegProfile(profileItem);
			}
		}

		/// <summary>
		/// Returns XML serialized segmentation target group.
		/// </summary>
		/// <param name="targetGroupItem">Folder item containing a segmentation target group.</param>
		/// <returns>A string containing XML serialized segmentation target group.</returns>
		public string OpenSegTargetGroup(esriFolderItem targetGroupItem)
		{
			try
			{
				return Client.OpenSegTargetGroup(targetGroupItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				return Client.OpenSegTargetGroup(targetGroupItem);
			}
		}

		/// <summary>
		/// Renames an existing project.
		/// </summary>
		/// <remarks>If the source project is absent or the target project already exists, this method
		/// throws an exception. This method also throws an exception in attempt to rename a project
		/// to itself.</remarks>
		/// <param name="workspaceName">Workspace name.</param>
		/// <param name="projectName">Project name.</param>
		/// <param name="newProjectName">New project name.</param>
		public void RenameProject(string workspaceName, string projectName, string newProjectName)
		{
			try
			{
				Client.RenameProject(workspaceName, projectName, newProjectName);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				Client.RenameProject(workspaceName, projectName, newProjectName);
			}
		}

		/// <summary>
		/// Copies a workspace to another workspace.
		/// </summary>
		/// <remarks>
		/// If the source workspace is absent or the target workspace already exists, this method
		/// has no effect.
		/// </remarks>
		/// <param name="workspaceName">Workspace name.</param>
		/// <param name="newWorkspaceName">New workspace name.</param>
		public void RenameWorkspace(string workspaceName, string newWorkspaceName)
		{
			try
			{
				Client.RenameWorkspace(workspaceName, newWorkspaceName);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				Client.RenameWorkspace(workspaceName, newWorkspaceName);
			}
		}

		/// <summary>
		/// Sets properties for a project.
		/// </summary>
		/// <param name="workspaceName">Workspace name.</param>
		/// <param name="projectName">Project name.</param>
		/// <param name="propertySet">Properties.</param>
		public void SetProjectProperties(string workspaceName, string projectName, PropertySet propertySet)
		{
			try
			{
				Client.SetProjectProperties(workspaceName, projectName, propertySet);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				Client.SetProjectProperties(workspaceName, projectName, propertySet);
			}
		}

		/// <summary>
		/// Uploads a zipped folder to the repository.
		/// </summary>
		/// <remarks>This method is usually used for uploading the contents of the "Custom&#160;Data" folder.</remarks>
		/// <param name="workspaceName">Workspace name.</param>
		/// <param name="projectName">Project name.</param>
		/// <param name="fileUrl">URL to a zip-archive with folder contents already uploaded to the ArcGIS Server virtual output directory.</param>
		/// <param name="overwriteAll">Overwrite any existing content.</param>
		/// <returns>True value if the upload is successful.</returns>
		public bool UploadData(string workspaceName, string projectName, string fileUrl, bool overwriteAll)
		{
			string[] results;

			try
			{
				results = Client.UploadData(workspaceName, projectName, fileUrl, overwriteAll);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				results = Client.UploadData(workspaceName, projectName, fileUrl, overwriteAll);
			}
			foreach (string result in results)
				if (!result.Equals("true", StringComparison.InvariantCultureIgnoreCase))
					return false;
			return true;
		}

		/// <summary>
		/// Uploads a custom feature set into the repository.
		/// </summary>
		/// <param name="featureSet">Custom feature layer.</param>
		/// <param name="outputAnalysisItem">Folder item to upload the custom feature layer.</param>
		public void UploadFeatureSet(RecordSet featureSet, esriFolderItem outputAnalysisItem)
		{
			UploadFeatureSetParameters parameters = new UploadFeatureSetParameters();
			parameters.FeatureSet = new DataLayer();
			parameters.FeatureSet.RecordSet = featureSet;
			parameters.IsFullErrorMessage = isFullErrorMessage;

			try
			{
				Client.UploadFeatureSet(parameters, outputAnalysisItem);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				Client.UploadFeatureSet(parameters, outputAnalysisItem);
			}
		}

		/// <summary>
		/// Uploads a shapefile into the repository.
		/// </summary>
		/// <remarks>
		/// <note type="caution">
		/// The <c>ShapefileFolderParameters</c> parameter is removed from this proxy-method.
		/// </note>
		/// </remarks>
		/// <param name="fileUrl">URL to a shapefile zip-archive already uploaded to ArcGIS Server virtual output directory.</param>
		/// <param name="outputItem">Configuration options for storing the shapefile in the repository.</param>
		/// <param name="shapefileParameters">Shapefile parameters.</param>
		public void UploadShapefile(string fileUrl, esriFolderItem outputItem, UploadShapefileParameters shapefileParameters)
		{
			try
			{
				Client.UploadShapefile(fileUrl, null, outputItem, shapefileParameters);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				Client.UploadShapefile(fileUrl, null, outputItem, shapefileParameters);
			}
		}

		/// <summary>
		/// Uploads external data in tabular form into the repository.
		/// </summary>
		/// <remarks>Supported table formats: XLS, XLSX, and DBF.</remarks>
		/// <param name="workspaceName">Workspace name.</param>
		/// <param name="projectName">Project name.</param>
		/// <param name="itemName">Name of project item to upload the data table.</param>
		/// <param name="fileUrl">URL to a table zip-archive already uploaded to ArcGIS Server virtual output directory.</param>
		/// <param name="overwriteAll">Overwrite any existing content.</param>
		public void UploadTable(string workspaceName, string projectName, string itemName, string fileUrl,
			bool overwriteAll)
		{
			try
			{
				Client.UploadTable(workspaceName, projectName, itemName, fileUrl, overwriteAll);
			}
			catch (WebException ex)
			{
				if (!IsTokenExpired(ex)) throw;
				UpdateToken();
				Client.UploadTable(workspaceName, projectName, itemName, fileUrl, overwriteAll);
			}
		}

		#endregion
	}
}
